
<!DOCTYPE html>
<html ng-app="Static">
<head>
    <meta charset="UTF-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <title>Learning Assemble, Step 7 - Custom Handlebars Helpers | James Wing</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta name="description" content="Part of a series exploring the Assemble static web site framework for Node.js" />

    

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/font-awesome/4.1.0/css/font-awesome.min.css"/>

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/css/bootstrap.min.css" />



<link rel="stylesheet" type="text/css" href="../assets/css/compiled/site.css">



<!--<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/highlight.js/8.6/styles/default.min.css">-->


<link href="../assets/prism/prism.css" rel="stylesheet" />

<!--[if lt IE 9]>
<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->

<script src="//ajax.googleapis.com/ajax/libs/jquery/1.10.2/jquery.min.js"></script>


</head>
<body id="blogpost" ng-controller="BaseController">



<header class="navbar navbar-inverse normal" role="banner">
    <div class="navbar-container">
        <div class="navbar-header">
            <button class="navbar-toggle" type="button" data-toggle="collapse" data-target=".bs-navbar-collapse">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a href="/index.html" class="navbar-brand">James Wing</a>
        </div>
        <nav class="collapse navbar-collapse bs-navbar-collapse" role="navigation">
            <ul class="nav navbar-nav navbar-right">
                

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


            </ul>
        </nav>
    </div>
</header>



<div class="layout-sidebar">
    <div class="row">
        <div class="col-sm-12 col-md-9 col-lg-8">
            

<div class="blogpost-wrapper">
    

    <div class="row">
        <div class="col-md-10 col-lg-10 post">
            <h1 class="title">
                <a href="">Learning Assemble, Step 7 - Custom Handlebars Helpers</a>
            </h1>
            <!--<div class="author">-->
                <!---->
                <!---->
                <!---->
                <!--2014-10-29 by James Wing-->
            <!--</div>-->
            <div class="date">
                2014-10-29
            </div>

            

            <div class="content">

                
                    <p>If you don&#x27;t like what Assemble does, you can probably change it with custom Handlebars Helpers.
The best part is that there are already many good open-source helpers around, either for direct use or for
study and modification.  We&#x27;ll look at how to use and author Handlebars helpers in this post.</p>
<ul>
<li>About Handlebars Helpers</li>
<li>Writing a Simple Helper</li>
<li>Writing a Block Helper for Navigation Links</li>
</ul>

                    <h2 id="about-handlebars-helpers">About Handlebars Helpers</h2>
<p>A Handlebars Helper is a Javascript function, registered with the Handlebars engine.
Formal distribution of helpers consists of installing them as libraries through NPM, just like the
helpers that come <a href="http://handlebarsjs.com/builtin_helpers.html">built-in to Handlebars</a>
and the many <a href="http://assemble.io/helpers/">Handlebars helpers provided by Assemble</a>.
Thankfully, it is also possible to install them as simple scripts in your project, as long as you do two things:</p>
<ol>
<li>Modify your assemble configuration to specify where helpers are found</li>
<li>Properly format the helper scripts</li>
</ol>
<h3 id="helper-references">Helper References</h3>
<p>I&#39;m hardly writing the definitive post on Handlebars helpers.  You would be well served to take a
look at the following references:</p>
<ul>
<li><a href="http://handlebarsjs.com/">Handlebars</a> - The mechanics of custom helpers are described briefly in the home page.</li>
<li><a href="http://handlebarsjs.com/block_helpers.html">Handlebars Block Helpers</a> - Nice concise and helpful documentation of the mechanics of block helpers, with samples.</li>
<li><a href="https://github.com/helpers">Assemble Handlebars Helpers</a> - This project has many helpers from the guys who wrote Assemble, and is a good place to look for ideas and sample code.</li>
<li><a href="https://github.com/ghost-town/grunt-init-helper">grunt-init-helper</a> - A good, quick way to get a properly formatted helper started.</li>
</ul>
<h2 id="writing-a-simple-helper">Writing a Simple Helper</h2>
<p>We&#39;ll write a simple &quot;Hello, World!&quot; helper first, continuing the spirit of this series.</p>
<h3 id="helper-configuration">Helper Configuration</h3>
<p>The first thing we need to do is modify the Assemble configuration in our Gruntfile to define where Assemble
should look for and register helpers:</p>
<pre class="language-javascript line-numbers" data-line="11" data-start="3"><code>
        assemble: {
            hello: {
                options: {
                    data: 'source/data/*.{json,yml}',
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs',
                    collections: [
                        { name: 'navTags', inflection: 'navTag' }
                    ],
                    helpers: ['source/helpers/**/*.js']
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.{hbs,md}', dest: 'output/' }
                ]
            }
        },
</code></pre>

<p>We should also modify the configuration for Watch to include our helper files.</p>
<pre class="language-javascript line-numbers" data-line="3" data-start="32"><code>
        watch: {
            templates: {
                files: ['source/templates/**/*.{hbs,md}','source/data/**/*.json','source/helpers/**/*.js'],
                tasks: ['assemble']
            },
            livereload: {
                options: {
                    livereload: '<%= connect.preview.options.livereload %>'
                },
                files: ['output/**.*']
            }
        }
</code></pre>

<h3 id="creating-the-hello-helper">Creating the Hello Helper</h3>
<p>I used <a href="https://github.com/ghost-town/grunt-init-helper">grunt-init-helper</a> to get started with some boilerplate,
then customized it into this awesome &quot;Hello, World!&quot; Handlebars helper.
I put the code in source/helpers/<strong>hello-helper.js</strong>:</p>
<pre><code class="language-javascript">(<span class="function"><span class="keyword">function</span><span class="params">()</span> {</span>
    module.exports.register = <span class="function"><span class="keyword">function</span><span class="params">(Handlebars, options)</span> {</span>

        Handlebars.registerHelper(<span class="string">'hello'</span>, <span class="function"><span class="keyword">function</span><span class="params">(name)</span> {</span>
            <span class="keyword">return</span> <span class="keyword">new</span> Handlebars.SafeString(<span class="string">"Hello, "</span> + name + <span class="string">"!"</span>);
        });

    };
}).call(<span class="keyword">this</span>);</code></pre>
<h3 id="testing-the-hello-helper">Testing the Hello Helper</h3>
<p>Now you can use this helper in a page with <code>{{hello &quot;World&quot;}}</code>.
For example, in our <strong>about.hbs</strong> page:
<!--[code language="html" highlight="11"]--></p>
<pre><code class="language-markup">---
title: &quot;About Us&quot;
navTags:
 - header
 - footer
navLabel: About
navSort: 70
---
&lt;h1&gt;{{ title }}&lt;/h1&gt;
&lt;p&gt;This is the About page content&lt;/p&gt;
{{hello &quot;World&quot;}}</code></pre>
<p>Our helper does some simple string concatenation, and &quot;Hello, World!&quot; is injected into the page.
<!--[code language="html" startline="29"]--></p>
<pre><code class="language-markup">&lt;h1&gt;About Us&lt;/h1&gt;
&lt;p&gt;This is the About page content&lt;/p&gt;
Hello, World!
&lt;hr/&gt;</code></pre>
<p>You might be underwhelmed by the hello helper.  But I hope you agree that it is easy to add new helper
logic and customize your Assemble site.</p>
<h2 id="writing-a-block-helper">Writing a Block Helper</h2>
<p>For added excitement, let&#39;s create our own helper to iterate over pages based on our navTag concept
introduced in an earlier post. The
<a href="http://handlebarsjs.com/block_helpers.html">Handlebars block helper docs</a> provide a starting
point for a basic iterator based on the <code>{{#each}}</code> helper:</p>
<pre><code class="language-javascript">Handlebars.registerHelper(<span class="string">'each'</span>, <span class="function"><span class="keyword">function</span><span class="params">(context, options)</span> {</span>
  <span class="keyword">var</span> ret = <span class="string">""</span>;

  <span class="keyword">for</span>(<span class="keyword">var</span> i=<span class="number">0</span>, j=context.length; i&lt;j; i++) {
    ret = ret + options.fn(context[i]);
  }

  <span class="keyword">return</span> ret;
});</code></pre>
<p>But we want to iterate through all of the pages associated with one tag in the navTags collection.
For example, all of the pages tagged &quot;header&quot;.  A first pass at source/helpers/<strong>nav-helpers.js</strong>
looks as follows:</p>
<pre><code class="language-javascript">(<span class="function"><span class="keyword">function</span><span class="params">()</span> {</span>
    <span class="keyword">var</span> _ = require(<span class="string">"lodash"</span>);
    module.exports.register = <span class="function"><span class="keyword">function</span><span class="params">(Handlebars, options)</span> {</span>
        Handlebars.registerHelper(<span class="string">"navPages"</span>, <span class="function"><span class="keyword">function</span><span class="params">(navTag, options)</span> {</span>
            <span class="keyword">var</span> result = <span class="string">""</span>;
            <span class="keyword">var</span> collectionItem = _.find(<span class="keyword">this</span>[<span class="string">"navTags"</span>], {<span class="string">"navTag"</span>: navTag});
            <span class="keyword">if</span> (collectionItem) {
                _.each(collectionItem.pages, <span class="function"><span class="keyword">function</span> <span class="params">(page, index, pages)</span> {</span>
                    result += options.fn(page);
                });
            }
            <span class="keyword">return</span> result;
        });
    };
}).call(<span class="keyword">this</span>);</code></pre>
<p>The hard part of this code is figuring out the data model of our navTags collection.
I discovered that through the use of <code>console.dir(this)</code> and then finding the tag data.
The relevant snippet of the data model is this:</p>
<pre><code class="language-javascript">  navTags:
   [ { navTag: <span class="string">'blog'</span>, pages: [Object] },
     { navTag: <span class="string">'footer'</span>, pages: [Object] },
     { navTag: <span class="string">'header'</span>, pages: [Object] } ],</code></pre>
<p>There is a top-level navTags array of item objects, where each item having an array of pages.
That matches what we know about collections.  The code I wrote above hard codes the &quot;navTags&quot; collection
and &quot;navTag&quot; item name, but something more generic could be written to traverse any collection, items, and pages.</p>
<p>To test this, let&#39;s put this in our <strong>header.hbs</strong>:</p>
<pre><code class="language-markup">&lt;div&gt;
    {{#navPages &quot;header&quot;}}
        &lt;a href=&quot;/{{relativeLink}}&quot;&gt;{{data.navLabel}}&lt;/a&gt;
    {{/navPages}}
&lt;/div&gt;</code></pre>
<p>This works, but it still has some of the problems we experienced in the earlier navigation post.
First, the sorting of these pages is not right.  Also, we want to separate the links with a pipe (&quot;|&quot;),
but there should be no pipe after the last link.</p>
<p>The sorting can be fixed by sorting our array of selected pages by the custom <code>navSort</code>
attribute before we render the content of the helper.  A revised <strong>nav-helpers.js</strong>:</p>
<pre class="language-javascript line-numbers" data-line="8,9,10,11" data-start="1"><code>
(function() {
    var _ = require("lodash");
    module.exports.register = function(Handlebars, options) {
        Handlebars.registerHelper("navPages", function(navTag, options) {
            var result = "";
            var collectionItem = _.find(this["navTags"], {"navTag": navTag});
            if (collectionItem) {
                var sortedPages = _.sortBy(collectionItem.pages, function (page, index, pages) {
                    return page.data.navSort || 0;
                });
                _.each(sortedPages, function (page, index, pages) {
                    result += options.fn(page);
                });
            }
            return result;
        });
    };
}).call(this);
</code></pre>

<p>To manage presentation of the list items, we&#39;ll add the zero-based index of where the page falls
in our nav list to the page data object.</p>
<pre class="language-javascript line-numbers" data-line="12" data-start="1"><code>
(function() {
    var _ = require("lodash");
    module.exports.register = function(Handlebars, options) {
        Handlebars.registerHelper("navPages", function(navTag, options) {
            var result = "";
            var collectionItem = _.find(this["navTags"], {"navTag": navTag});
            if (collectionItem) {
                var sortedPages = _.sortBy(collectionItem.pages, function (page, index, pages) {
                    return page.data.navSort || 0;
                });
                _.each(sortedPages, function (page, index, pages) {
                    page.navIndex = index;
                    result += options.fn(page);
                });
            }
            return result;
        });
    };
}).call(this);
</code></pre>

<p>We can use this in <strong>header.hbs</strong> similar to the way we once used the built-in <code>@index</code> variable.
At the time, we found that <code>@index</code> did not work without the context of filtering
applied to the list.  Here, we control that context.</p>
<pre><code class="language-markup">&lt;div&gt;
    {{#navPages &quot;header&quot;}}
        {{#if navIndex}} | {{/if}}
        &lt;a href=&quot;/{{relativeLink}}&quot;&gt;{{data.navLabel}}&lt;/a&gt;
    {{/navPages}}
&lt;/div&gt;</code></pre>
<p>I&#39;m not sure this is the best solution for navigation, but it&#39;s a bit better than our header was at
the start of this post. This doesn&#39;t feel particularly reusable to me, given the dependence on having
the navTags collection and YAML Front-Matter for the tagging, labeling, and sorting.
Still, I enjoyed being able to dive into the data and work with it in a custom helper.</p>
<p>For the code, please see the <a href="https://github.com/jvwing/learning-assemble">learning-assemble</a> repository
on GitHub.
The branch <a href="https://github.com/jvwing/learning-assemble/tree/step07-handlebars-helpers">step07-handlebars-helpers</a>
contains the completed code from this post.</p>

                

                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                        <div class="blog-post-next">
                            Next:
                            <a href="/learning-assemble/step-8-debugging-assemble.html">
                                Learning Assemble, Step 8 - Debugging Assemble <i class="fa fa-lg fa-arrow-right"></i>
                            </a>
                        </div>
                    
                
                    
                
                    
                
                    
                
                    
                

            </div>

            <div class="share">
                <a href="https://twitter.com/share" class="twitter-share-button" data-text="Learning Assemble, Step 7 - Custom Handlebars Helpers" data-via="jvwing" data-size="large">Tweet</a>
                <script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+'://platform.twitter.com/widgets.js';fjs.parentNode.insertBefore(js,fjs);}}(document, 'script', 'twitter-wjs');</script>
            </div>

            <hr/>
            <div class="other-posts">
                <h4>You might also enjoy:</h4>
                <ul>
                    
                        <li><a href="/learning-assemble/step-1-hello-assemble.html">Learning Assemble, Step 1 - Hello, Assemble!</a></li>
                    
                        <li><a href="/learning-assemble/step-2-basic-templating.html">Learning Assemble, Step 2 - Basic Templating</a></li>
                    
                        <li><a href="/learning-assemble/step-3-grunt-workflow.html">Learning Assemble, Step 3 - Grunt Workflow</a></li>
                    
                        <li><a href="/learning-assemble/step-4-data.html">Learning Assemble, Step 4 - Data</a></li>
                    
                        <li><a href="/learning-assemble/step-5-page-navigation.html">Learning Assemble, Step 5 - Page Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-6-blogging-with-markdown.html">Learning Assemble, Step 6 - Blogging with Markdown</a></li>
                    
                        <li><a href="/learning-assemble/step-7-custom-handlebars-helpers.html">Learning Assemble, Step 7 - Custom Handlebars Helpers</a></li>
                    
                        <li><a href="/learning-assemble/step-8-debugging-assemble.html">Learning Assemble, Step 8 - Debugging Assemble</a></li>
                    
                        <li><a href="/learning-assemble/step-9-sitemap.html">Learning Assemble, Step 9 - Sitemap</a></li>
                    
                        <li><a href="/learning-assemble/step-10-pages-from-data.html">Learning Assemble, Step 10 - Pages from Data</a></li>
                    
                        <li><a href="/learning-assemble/step-11-more-navigation.html">Learning Assemble, Step 11 - More Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-12-navigation-helper.html">Learning Assemble, Step 12 - Navigation Helper</a></li>
                    
                </ul>
            </div>

            <div class="comments">
                <div id="disqus_thread"></div>
<script type="text/javascript">
    /* * * CONFIGURATION VARIABLES * * */
    var disqus_shortname = 'jameswingnet';

    /* * * DON'T EDIT BELOW THIS LINE * * */
    (function() {
        var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
        dsq.src = '//' + disqus_shortname + '.disqus.com/embed.js';
        (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
    })();
</script>
<noscript>Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript" rel="nofollow">comments powered by Disqus.</a></noscript>
            </div>

        </div>

    </div>

</div>

        </div>
        <div class="col-md-3 col-lg-4 sidebar">
            <div class="posts-list-brief">
    <h3 class="sidebar-heading">Recent</h3>
    <ul>
        
            
        
            
                
                    <li>
                        
                            <a href="/aws/query-cloudtrail-events-with-athena.html">Query CloudTrail Events with Athena</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/nifi-internal-fields.html">Hidden Fields in Apache NiFi FlowFiles</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/first-look-at-apache-nifi.html">First Look at Apache NiFi</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/monitoring/monitoring-advice-distilled.html">Monitoring Advice Distilled</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/aws/using-codecommit-and-git-credentials.html">Using CodeCommit and GitHub Credential Helpers</a>
                        
                    </li>
                
            
        
            
        
            
        
            
        
            
        
    </ul>
</div>
            <hr/>
            <div class="ads">
    <!--<h3 class="sidebar-heading">Featured</h3>-->
    <div class="ad">
        <a href="http://prontolyzer.jameswing.net/">
            <img class="thumbnail" src="/assets/ads/prontolyzer-thumb.png" />
            <h4 class="title">Prontolyzer</h4>
            <p class="description">
                Interactive mapping and exploration of bike trip data for the 2015 Pronto Cycle Share Data Challenge.
                
            </p>
        </a>
    </div>
    <hr/>
    <div class="ad">
        <a href="/troubleshooting-chart/introducing-the-troubleshooting-chart.html">
            <img class="thumbnail" src="/assets/posts/troubleshooting-chart/troubleshootingchartannotated-thumbnail.png" />
            <h4 class="title">The Troubleshooting Chart</h4>
            <p class="description">
                Level up your troubleshooting with simple data analysis and visualizations.
                A thrill-ride for Ops types.
            </p>
        </a>
    </div>
</div>
        </div>
    </div>
</div>




<div id="footer">
    <div class="container">
        <div class="row">
            <div class="col-sm-6 copyright">
                © 2014-2017 James Wing. All Rights Reserved.
            </div>
            <div class="col-sm-6 menu">
                <ul>
                    

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


                </ul>
            </div>
        </div>
    </div>
</div>






<script src="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/js/bootstrap.min.js"></script>

<script>
    (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
        (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
            m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
    })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
    var gaId = 'UA-6573';
    gaId += '6738-1';
    ga('create', gaId, 'auto');
    ga('send', 'pageview');

</script>









<script src="../assets/prism/prism.js"></script>


</body>
</html>